@@ -1,10 +1,12 @@
 """Classes for describing plugins"""
 import inspect
+from dataclasses import dataclass, field
 from inspect import _empty
 from typing import Optional, List, Type, Any
 
 from cmem_plugin_base.dataintegration.plugins import WorkflowPlugin, TransformPlugin
-from cmem_plugin_base.dataintegration.types import ParameterType, get_param_type
+from cmem_plugin_base.dataintegration.types import ParameterType, get_param_type, \
+    PluginContextParameterType
 from cmem_plugin_base.dataintegration.utils import generate_id
 
 
@@ -21,6 +23,7 @@ class PluginParameter:
         Will be inferred from the plugin automatically.
     :param advanced: True, if this is an advanced parameter that should only be
         changed by experienced users
+    :param visible: If true, the parameter will be displayed to the user in the UI.
     """
 
     def __init__(
@@ -31,6 +34,7 @@ class PluginParameter:
         param_type: Optional[ParameterType] = None,
         default_value: Optional[Any] = None,
         advanced: bool = False,
+        visible: bool = True
     ) -> None:
         self.name = name
         self.label = label
@@ -38,6 +42,7 @@ class PluginParameter:
         self.param_type = param_type
         self.default_value = default_value
         self.advanced = advanced
+        self.visible = visible
 
 
 class PluginDescription:
@@ -71,7 +76,7 @@ class PluginDescription:
             self.plugin_type = "TransformPlugin"
         else:
             raise ValueError(
-                f"Class {plugin_class.__name__} does not implement a supported"
+                f"Class {plugin_class.__name__} does not implement a supported "
                 f"plugin base class (e.g., WorkflowPlugin)."
             )
 
@@ -97,6 +102,34 @@ class PluginDescription:
             self.parameters = parameters
 
 
+@dataclass
+class PluginDiscoveryError:
+    """Generated if a plugin package could not be loaded."""
+
+    package_name: str
+    """The name of the package that failed to be loaded."""
+
+    error_message: str
+    """The error message"""
+
+    error_type: str
+    """The name of the raised exception"""
+
+    stack_trace: str
+    """The stack trace of the raised exception"""
+
+
+@dataclass
+class PluginDiscoveryResult:
+    """Result of running a plugin discovery"""
+
+    plugins: list[PluginDescription] = field(default_factory=list)
+    """The list of discovered plugins"""
+
+    errors: list[PluginDiscoveryError] = field(default_factory=list)
+    """Errors that occurred during discovering plugins."""
+
+
 class Categories:
     """A list of common plugin categories. At the moment, in the UI,
     categories are only utilized for rule operators, such as transform plugins."""
@@ -196,6 +229,12 @@ class Plugin:
                 sig_param = sig.parameters[name]
                 if param.param_type is None:
                     param.param_type = get_param_type(sig_param)
+
+                # Special handling of PluginContext parameter
+                if isinstance(param.param_type, PluginContextParameterType):
+                    param.visible = False  # Should never be visible in the UI
+                    param.default_value = ""  # dummy value
+
                 if param.default_value is None and sig_param.default != _empty:
                     param.default_value = sig_param.default
                 params.append(param)
